@@ -12,7 +12,7 @@ from typing import (
     Union,
     Literal,
 )
-from sentry_kafka_schemas.types import Schema
+from sentry_kafka_schemas.types import Schema, Example
 from pathlib import Path
 from yaml import safe_load
 
@@ -31,11 +31,16 @@ TopicSchema = TypedDict(
         "type": Union[Literal["json"]],
         "compatibility_mode": Union[Literal["none"], Literal["backward"]],
         "resource": str,
+        "examples": Sequence[str],
     },
 )
 
 TopicData = TypedDict("TopicData", {"version": int, "schemas": Sequence[TopicSchema]})
 
+_TOPICS_PATH = Path.joinpath(Path(__file__).parent, "topics")
+_SCHEMAS_PATH = Path.joinpath(Path(__file__).parent, "schemas")
+_EXAMPLES_PATH = Path.joinpath(Path(__file__).parent, "examples")
+
 
 def _list_topics() -> Iterable[str]:
     """
@@ -43,7 +48,7 @@ def _list_topics() -> Iterable[str]:
 
     This is not yet stable API, just internally used by code generation.
     """
-    for file in os.listdir(Path.joinpath(Path(__file__).parent, "topics")):
+    for file in os.listdir(_TOPICS_PATH):
         assert file.endswith(".yaml")
         yield file[: -len(".yaml")]
 
@@ -54,7 +59,7 @@ def _get_topic(topic: str) -> TopicData:
 
     This is not yet stable API, just internally used by code generation.
     """
-    topic_path = Path.joinpath(Path(__file__).parent, "topics", f"{topic}.yaml")
+    topic_path = Path.joinpath(_TOPICS_PATH, f"{topic}.yaml")
 
     try:
         with open(topic_path) as f:
@@ -77,7 +82,6 @@ def get_schema(topic: str, version: Optional[int] = None) -> Schema:
     schema_key = (topic, version)
 
     if schema_key not in __TOPIC_TO_SCHEMA:
-
         try:
             topic_data = _get_topic(topic)
 
@@ -99,9 +103,7 @@ def get_schema(topic: str, version: Optional[int] = None) -> Schema:
             __TOPIC_TO_SCHEMA[schema_key] = None
             raise
 
-        resource_path = Path.joinpath(
-            Path(__file__).parent, "schemas", schema_metadata["resource"]
-        )
+        resource_path = Path.joinpath(_SCHEMAS_PATH, schema_metadata["resource"])
         with open(resource_path) as f:
             json_schema = rapidjson.load(f)
         schema: Schema = {
@@ -110,6 +112,7 @@ def get_schema(topic: str, version: Optional[int] = None) -> Schema:
             "compatibility_mode": schema_metadata["compatibility_mode"],
             "schema": json_schema,
             "schema_filepath": str(resource_path),
+            "examples": schema_metadata["examples"],
         }
 
         __TOPIC_TO_SCHEMA[schema_key] = schema
@@ -121,3 +124,31 @@ def get_schema(topic: str, version: Optional[int] = None) -> Schema:
     cached_schema = __TOPIC_TO_SCHEMA[schema_key]
     assert cached_schema is not None
     return cached_schema
+
+
+def iter_examples(topic: str, version: Optional[int] = None) -> Iterable[Example]:
+    """
+    Yield filepaths to all example payloads for a specific schema version, or
+    the latest version if no version is provided.
+
+    Parameters are the same as for `get_schema`. If a matching schema can't be
+    found, raise `SchemaNotFound`.
+
+    This is currently a separate function, not a method on a Schema class,
+    because we might want to split out examples from this package at some point.
+    """
+    schema = get_schema(topic, version)
+
+    for example_entry in schema["examples"]:
+        example_path = Path.joinpath(_EXAMPLES_PATH, example_entry)
+        if os.path.isfile(example_path):
+            yield Example(
+                _examples_basepath=_EXAMPLES_PATH,
+                path=example_path,
+            )
+        else:
+            for example_subpath in os.listdir(example_path):
+                yield Example(
+                    _examples_basepath=_EXAMPLES_PATH,
+                    path=Path.joinpath(example_path, example_subpath),
+                )
